import React, { createContext, useContext, useState } from "react";
import * as api from "../lib/api";
import { toast } from "react-toastify";
export const TodoContext = createContext();
export const useAppContext = () => useContext(TodoContext);

export function TodoProvider({ children }) {
  const [todos, setTodos] = useState([]);
  const [isLoading, setIsLoading] = useState(false);
  //POST METHOD
  async function addTodo({ title }) {
    try {
      //default isCompleted is false
      const response = await api.addTodo({ title, isCompleted: false });
      setTodos([...todos, response]);
    } catch (err) {
      toast.error(err.message);
    }
  }
  //UPDATE METHOD
  async function updateTodo({ id, title, isCompleted }) {
    try {
      const response = await api.updateTodo({ id, title, isCompleted });

      const data = await response.json();
      const result = todos.map((e) => (e.id === id ? data : e));
      setTodos(result);
    } catch (err) {
      toast.error(err.message);
    }
  }

  //GET METHOD
  async function getTodos() {
    try {
      const response = await api.getTodos();
      setTodos(response);
      setIsLoading(true);
      return response;
    } catch (err) {
      toast.error(err.message);
    }
  }

  //DELETE METHOD
  async function deleteTodo(id) {
    try {
      await api.deleteTodo(id);
      const result = todos.filter((e) => e.id !== id);
      setTodos(result);
    } catch (err) {
      toast.error(err.message);
    }
  }

  const value = {
    todos,
    isLoading,
    addTodo,
    getTodos,
    deleteTodo,
    updateTodo,
  };
  return <TodoContext.Provider value={value}>{children}</TodoContext.Provider>;
}
