import { FaTimesCircle } from "react-icons/fa";
import Checkbox from "../../base/checkbox";
import "./task.css";
import { useEffect, useRef, useState, useCallback } from "react";
import { toast } from "react-toastify";

function Task({ id = 1, title, isCompleted, onChange, onDelete }) {
  const [debouncetitle, setDebounceTitle] = useState(title);
  const [ischeck, setIsCheck] = useState(isCompleted);
  const textarea_ref = useRef(null);
  //debounce
  useEffect(() => {
    //checking the content height and set on page load

    // Optional: To ensure the textarea resizes initially if there's content
    const scrollHeight = textarea_ref.current.scrollHeight - 4;
    textarea_ref.current.style.height = `${scrollHeight}px`;
    //after 2 second it will execute
    const handler = setTimeout(() => {
      if (textarea_ref.current.value !== title) {
        onChange({ id, title: debouncetitle });
        // toast.success("Task updated");
      }
    }, 2000);

    return () => {
      clearTimeout(handler);
    };
  }, [debouncetitle]);

  const handleDelete = (e) => {
    e.preventDefault();
    const dialog = window.confirm("Do you want to delete?");
    if (dialog) {
      onDelete({ id });
      toast.info("Task deleted");
      return true;
    }
    return false;
  };

  // (useCallback)---> to avoid calling the checkbox component without any action applied on checkbox
  const handleChange = useCallback(() => {
    setIsCheck(!ischeck);
    onChange({ id, isCompleted: !ischeck });
    // toast(`${!ischeck ? "Task completed" : "Task not completed"}`, "success");
  }, [ischeck]);

  const handleContent = (e) => {
    setDebounceTitle(e.target.value);
    const textareaRef = textarea_ref.current;
    textareaRef.addEventListener("input", function () {
      this.style.height = "25px"; // Reset the height to auto to calculate the correct scrollHeight
      this.style.overflow = "hidden";

      // Check if the textarea is empty or null and set its height accordingly
      if (textareaRef.value === "" || textareaRef.value === null) {
        this.style.height = "25px"; // Minimum height of 25px when empty
      } else {
        // Calculate the new scrollHeight and set the height
        const newScrollHeight = Math.max(this.scrollHeight - 4, 25); // Minimum height of 25px
        this.style.height = newScrollHeight + "px";
      }
    });
    textareaRef.addEventListener("keyup", function (event) {
      if (
        event.key === "Backspace" &&
        (textareaRef.value === "" || textareaRef.value === null)
      ) {
        this.style.height = "25px"; // Minimum height of 25px when empty
      } else {
        console.log("dkod");
        this.style.height = "25px";
        const newScrollHeight = this.scrollHeight - 4; // Minimum height of 25px
        this.style.height = newScrollHeight + "px";
      }
    });

    // textareaRef.addEventListener("input", function () {
    //   this.style.height = "25px"; // Reset the height to auto to calculate the correct scrollHeight
    //   this.style.overflow = "hidden";

    //   // Check if the textarea is empty or null and set its height accordingly

    //   // Calculate the new scrollHeight and set the height
    //   const newScrollHeight = Math.max(this.scrollHeight - 4, 25); // Minimum height of 25px
    //   this.style.height = newScrollHeight + "px";
    // });
    // textareaRef.addEventListener("keyup", function (event) {
    //   if (
    //     event.key === "Backspace" &&
    //     (textareaRef.value === "" || textareaRef.value === null)
    //   ) {
    //     this.style.height = "29px"; // Minimum height of 25px when empty
    //   }
    //   else{

    //   }
    // });
  };

  return (
    <div className="task-section">
      <div className="task">
        <Checkbox
          onChange={handleChange}
          checked={ischeck}
          // disabled={ischeck}
        />

        <textarea
          ref={textarea_ref}
          className={`${ischeck ? "strike-through" : ""}`}
          value={debouncetitle}
          onChange={handleContent}

          // disabled={ischeck}
        />
      </div>
      <FaTimesCircle className="fa-close" onClick={handleDelete} />
    </div>
  );
}

export default Task;
